import QtQuick 2.0
import QtQuick.XmlListModel 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.Popups 0.1
import Ubuntu.Components.ListItems 0.1 as ListItem
import "datetime.js" as Datetime
import "utils.js" as Utils

Item {
    id: tabRoot
    anchors.fill: parent

    property real margins: parent.margins
    property string coreUrl: "http://journeyplanner.tfl.gov.uk/user/XML_TRIP_REQUEST2"

    Column {
        id: searchQuery

        anchors {
            top: parent.top
            left: parent.left
            right: parent.right
            margins: tabRoot.margins
        }
        spacing: units.gu(1)

        Grid {
            columns: 3
            spacing: units.gu(1)

            Label {
                text: i18n.tr("From")
            }
            TextField {
                id: originInput
                KeyNavigation.tab: destinationInput
            }
            Button {
                id: originPopoverTrigger
                text: "..."
                enabled: originModel.count > 1

                onClicked: {
                    PopupUtils.open(originPopover)
                }
            }

            Label {
                text: i18n.tr("To")
            }
            TextField {
                id: destinationInput
                KeyNavigation.backtab: originInput
            }
            Button {
                id: destinationPopoverTrigger
                text: "..."
                enabled: destinationModel.count > 1

                onClicked: {
                    PopupUtils.open(destinationPopover)
                }
            }
        }

        Row {
            spacing: units.gu(1)

            Button {
                id: searchButton
                text: i18n.tr("Search")
                width: units.gu(12)

                property bool busy: false

                onClicked: {
                    searchButton.busy = true
                    var now = new Date()

                    var doc = new XMLHttpRequest();
                    doc.onreadystatechange = function() {
                        if (doc.readyState === XMLHttpRequest.DONE) {
                            //console.log(doc.responseText)
                            tripModel.xml = doc.responseText
                            tripModel.reload()
                            searchButton.busy = false
                        }
                    }
                    doc.open("GET", tabRoot.coreUrl +
                             '?language=en&sessionID=0&type_origin=' +
                             Utils.inferInputType(originInput.text) +
                             '&name_origin=' +
                             originInput.text +
                             '&type_destination=' +
                             Utils.inferInputType(destinationInput.text) +
                             '&name_destination=' +
                             destinationInput.text +
                             '&itdDate=' + Datetime.iso8601DateString(now) +
                             '&itdTime=' + Datetime.iso8601TimeString(now))
                    doc.send()
                }
            }

            ActivityIndicator {
                running: searchButton.busy === true || tripModel.status === XmlListModel.Loading ||
                         originModel.status === XmlListModel.Loading || destinationModel.status === XmlListModel.Loading
            }
        }
    }

    Item {
        Rectangle {
            color: "white"
            anchors.fill: parent
        }

        anchors {
            top: searchQuery.bottom
            left: parent.left
            right: parent.right
            bottom: parent.bottom
            topMargin: tabRoot.margins
        }

        PageStack {
            id: searchResults
            anchors.fill: parent

            Component.onCompleted: push(tripPage)

            Page {
                id: tripPage
                title: "Trip"

                ListView {
                    id: tripList
                    anchors.fill: parent
                    model: tripModel
                    delegate: tripDelegate
                }
            }

            Page {
                id: journeyPage
                title: "Journey"

                ListView {
                    id: journeyList
                    anchors.fill: parent
                    model: journeyModel
                    delegate: journeyDelegate
                }
            }
        }
    }


    XmlListModel {
        id: tripModel
        xml: ""
        query: "/itdRequest/itdTripRequest/itdItinerary/itdRouteList/itdRoute"

        XmlRole {
            name: "changes"
            query: "@changes/string()"
        }
        XmlRole {
            name: "duration"
            query: "@publicDuration/string()"
        }
        XmlRole {
            name: "departHour"
            query: "itdPartialRouteList/itdPartialRoute[1]/itdPoint[1]/itdDateTime/itdTime/@hour/string()"
        }
        XmlRole {
            name: "departMinute"
            query: "itdPartialRouteList/itdPartialRoute[1]/itdPoint[1]/itdDateTime/itdTime/@minute/string()"
        }
        XmlRole {
            name: "routeTripIndex"
            query: "@routeTripIndex/string()"
        }

        onStatusChanged: {
            if (status === XmlListModel.Loading) {
                originModel.reload()
                destinationModel.reload()
            }
        }
    }

    Component {
        id: tripDelegate

        ListItem.Empty {
            Row {
                spacing: units.gu(2)

                Label {
                    text: Datetime.displayTimeString(departHour, departMinute)
                    fontSize: "x-large"
                }
                Grid {
                    columns: 2
                    spacing: units.gu(1)
                    Label {
                        text: "Changes"
                    }
                    Label {
                        text: changes
                    }
                    Label {
                        text: "Duration"
                    }
                    Label {
                        text: duration
                    }
                }
            }

            onClicked: {
                searchResults.push(journeyPage)
                journeyModel.pageIndex = routeTripIndex
                journeyModel.reload()
            }
        }
    }

    XmlListModel {
        id: journeyModel
        xml: tripModel.xml
        property int pageIndex
        query: "/itdRequest/itdTripRequest/itdItinerary/itdRouteList/itdRoute["+pageIndex+"]/itdPartialRouteList/itdPartialRoute"

        XmlRole {
            name: "departLocation"
            query: "itdPoint[1]/@name/string()"
        }
        XmlRole {
            name: "departHour"
            query: "itdPoint[1]/itdDateTime/itdTime/@hour/string()"
        }
        XmlRole {
            name: "departMinute"
            query: "itdPoint[1]/itdDateTime/itdTime/@minute/string()"
        }
        XmlRole {
            name: "arriveLocation"
            query: "itdPoint[2]/@name/string()"
        }
        XmlRole {
            name: "arriveHour"
            query: "itdPoint[2]/itdDateTime/itdTime/@hour/string()"
        }
        XmlRole {
            name: "arriveMinute"
            query: "itdPoint[2]/itdDateTime/itdTime/@minute/string()"
        }
        XmlRole {
            name: "transport"
            query: "itdMeansOfTransport/@productName/string()"
        }
        XmlRole {
            name: "line"
            query: "itdMeansOfTransport/@shortname/string()"
        }
    }

    Component {
        id: journeyDelegate

        ListItem.Empty {
            height: journeyLegData.height + tabRoot.margins
            Row {
                spacing: units.gu(2)

                Grid {
                    spacing: units.gu(1)
                    columns: 1

                    Label {
                        text: Datetime.displayTimeString(departHour, departMinute)
                        fontSize: "large"
                    }
                    Label {
                        text: Datetime.displayTimeString(arriveHour, arriveMinute)
                        fontSize: "large"
                    }
                }

                Grid {
                    id: journeyLegData
                    spacing: units.gu(1)
                    columns: 2

                    Label {
                        text: i18n.tr("From")
                    }
                    Label {
                        text: departLocation
                    }
                    Label {
                        text: i18n.tr("To")
                    }
                    Label {
                        text: arriveLocation
                    }
                    Label {
                        text: i18n.tr("By")
                    }
                    Label {
                        text: transport + " (" + line + ")"
                    }
                }
            }
        }
    }

    XmlListModel {
        id: originModel
        xml: tripModel.xml
        query: "/itdRequest/itdTripRequest/itdOdv[@usage='origin']/itdOdvName/odvNameElem"

        XmlRole {
            name: "stopId"
            query: "@stopID/string()"
        }

        XmlRole {
            name: "stopName"
            query: "string()"
        }

        onStatusChanged: {
            if (status === XmlListModel.Ready) {
                if (count == 1) {
                    originInput.text = get(0).stopName
                }
            }
        }
    }

    Component {
        id: originPopover

        Popover {
            id: popover

            ListView {
                clip: true
                anchors {
                    left: parent.left
                    right: parent.right
                    top: parent.top
                }
                model: originModel
                height: units.gu(50)
                delegate: ListItem.Standard {
                    text: stopName
                    onClicked: {
                        originInput.text = stopName
                        PopupUtils.close(popover)
                    }
                }
            }

        }
    }

    XmlListModel {
        id: destinationModel
        xml: tripModel.xml
        query: "/itdRequest/itdTripRequest/itdOdv[@usage='destination']/itdOdvName/odvNameElem"

        XmlRole {
            name: "stopId"
            query: "@stopID/string()"
        }

        XmlRole {
            name: "stopName"
            query: "string()"
        }

        onStatusChanged: {
            if (status === XmlListModel.Ready) {
                if (count == 1) {
                    destinationInput.text = get(0).stopName
                }
            }
        }
    }

    Component {
        id: destinationPopover

        Popover {
            id: popover

            ListView {
                clip: true
                anchors {
                    left: parent.left
                    right: parent.right
                    top: parent.top
                }
                model: destinationModel
                height: units.gu(50)
                delegate: ListItem.Standard {
                    text: stopName
                    onClicked: {
                        destinationInput.text = stopName
                        PopupUtils.close(popover)
                    }
                }
            }

        }
    }
}
